#!/usr/bin/python 

import xmlrpclib
import socket
import sys
import os
import time
import stat

ServerExec  = 'perl /usr/local/bin/vulcan_server.pl'
PidDir      = '/tmp'
PidPrefix   = 'vulcan_'
PidSuffix   = '.pid'
PortRange   = range(13600,13800)

def get_next_port(port=None):
    """ return next pid number and pidfile for an unused connection"""

    def tryport(p,reuse=False):
        pidfile = os.path.join(PidDir,"%s%i%s" % (PidPrefix,p, PidSuffix))
        try:
            mtime = os.stat(pidfile)[stat.ST_MTIME]
        except OSError:  # file does not exst -- use this PID
            fx = open(pidfile,'w')
            return (p,pidfile)
        if reuse or (time.time() - mtime) > 1800:  # reuse old connection
            return (p,pidfile)
        return (None,None)

    # if an explicit port is requested, try that, even if we need to reuse one
    # (this is useful for debugging)
    if port is not None:
        return tryport(port,reuse=True)
        
    # if no port is specified, try the pool
    for p in PortRange:
        p,fpid = tryport(p)
        if p is not None: return (p,fpid)
        
    return (None,None) #  pool exhausted

def purge_connections(stale_age=7200,verbose=True):
    """ kill Vulcan Server processes and remove Pid files that are older than stale_age (in seconds)

    relies on linux/bsd-like  ps and kill system utilities.
    
    """
    purgefile = '/tmp/vulcan_purge.dat'
    pidcmd   = "ps -eo pid,args | grep 'perl'| grep '%%i' > %s 2>&1" % purgefile
    killcmd  = "kill -9 %i"

    if verbose:
        print 'Current Active Vulcan Ports'
        print 'Port     Time since last use (sec)'
    
    for fname in os.listdir(PidDir):
        if fname.startswith(PidPrefix) and fname.endswith(PidSuffix):
            modtime = os.stat(os.path.join(PidDir,fname))[stat.ST_MTIME]
            try:
                port = int(fname[len(PidPrefix):-4])
            except:
                if verbose:
                    print 'error getting port for file ', fname
                continue
            age = time.time()-modtime

            if verbose:
                print "  %i    %.2f" % (port, age)

            if age > stale_age:
                os.system(pidcmd % port)
                time.sleep(1.0)
                for i in open(purgefile).readlines():
                    try:
                        pid = int(i[:-1].strip().split()[0])
                        os.system(killcmd % pid)
                        os.unlink(os.path.join(PidDir,fname))
                    except:
                        if verbose:
                            print 'error with kill for pid/port ', pid, port
    try:
        os.unlink(purgefile)
    except OSError:
        pass

    
        
class VulcanClient:
    def __init__(self,host='localhost',port=None,purge=False):
        self.server = None
        self.kws = {}
        self.state_kws = {'element':None,'resource':None}
        self.available = None
        self.scattering = None

        if purge: purge_connections(verbose=False)
        
        self.port,self.pidfile = get_next_port(port=port)
        if self.port is None:
            raise IOError, 'No Ports available for VulcanServer'
        self.connect(host=host)
        
    def touch_pid(self):
        "touch pidfile, setting its modified time to now"
        if self.pidfile is not None:
            try:
                os.utime(self.pidfile,None)
            except OSError:
                pass
    
    def connect(self,host='localhost',spawn=True):
        "connect to a Vulcan Server, possibly spawning a new one."
        self.host = str(host)
        try:
            self.server = xmlrpclib.Server("http://%s:%i" % (self.host,self.port))
            self.rpcdata = self.server.available()
        except socket.error:
            print 'Starting Vulcan Server seems to not be running on port ', self.port
            self.server = None

            try:
                os.system('%s -p %i &' % (ServerExec, self.port))
                
                time.sleep(1.2)
                self.server = xmlrpclib.Server("http://%s:%i" % (self.host,self.port))
                self.rpcdata = self.server.available()
            except:
                
                print 'Cannot start Vulcan Server'
                self.server = None
                
        if self.server is not None:
            self.available = self.server.available()
            if 'None' in self.available: self.available.remove('None')

            self.scattering = self.server.scattering()
            if 'None' in self.scattering: self.scattering.remove('None')            

    def _prep(self,**kw):
        " prepare arguments for XML-RPC call"
        
        if self.server is None: self.connect()

        def _kwargs(**kw):  return kw

        args = _kwargs(**kw)
        for k,v in args.items():
            if v is None: args.pop(k)
        
        self.kws = args
        # use 'element' and 'resource' as "state keywords":
        # if explicitly set, use and save for next use.
        # if not set, look up and use saved value.
        for key in ('element','resource'):
            if self.kws.has_key(key):
                self.state_kws[key] = self.kws[key]
            elif self.state_kws[key] not in (None,''):
                self.kws[key] = self.state_kws[key]


        # before returning, touch pid file, signaling that this
        # port / pid are still in use
        self.touch_pid()

        return self.kws
    
    def has(self,element=None, **kw):
        """return True if element is in the current resource
        >>> x = VC.has(element='Mo', resource='McMaster')
        """        
        return (1 == self.server.has(self._prep(element=element,**kw)))

    def weight(self,element=None, **kw):
        """return elements weight
        >>>x = VC.weight(element='Mo')
        """
        return self.server.weight(self._prep(element=element, **kw))

    def density(self,element=None, **kw):
        """return elements density (most common form of pure element)
        >>>x = VC.density(element='Mo')
        """        
        return self.server.density(self._prep(element=element, **kw))

    def energy(self,element=None, **kw):
        """return energy in eV of an element's edge or emission line

        >>>x = VC.energy(element='Cu', edge='K')
        >>>x = VC.energy(element='Cu', line='Kalpha1')
        
        Edges are strings such as 'K', 'L3', 'M5', etc
        Lines can use Siegbahn ('Lbeta3'), shortened Siegbahn ('Lb3')
        or IUPAC ('L1-M3') notation.
        """
        return self.server.energy(self._prep(element=element, **kw))    



    def next_energy(self,element=None,edge=None,atoms=None,**kw):
        """ Given an element's edge and a list of atoms, returns the next highest
        edge that will be seen for that list of atoms.
        
        return (element, edge, energy) for the next edge (energy in eV)
        """
        if atoms is None or edge is None:
            print 'need an edge and list of atoms'
        self._prep(element=element,edge=edge,atoms= tuple(atoms), **kw)
        return tuple(self.server.next_energy(self.kws))

    def conversion(self,**kw):
        """Return the coeffient for converting cross-sections between barns to
        grams per square centimeter"""
        return self.server.conversion(self._prep(**kw))
    
    def siegbahn(self,**kw):
        """Return a line symbol in Siegbahn form.
        Input can be Siegbahn (Lbeta3), shortened Siegbahn (Lb3), or IUPAC (L1-M3).
        
        >>> print VC.siegbahn(line='lb3')
        Lbeta3
        """
        return self.server.siegbahn(self._prep(**kw))

    def iupac(self,**kw):
        """Return a line symbol in IUPAC form.
        Input can be Siegbahn (Lbeta3), shortened Siegbahn (Lb3), or IUPAC (L1-M3).
        
        >>> print VC.iupac(line='lb3')
        L1-M3
        """
        return self.server.iupac(self._prep(**kw))
    
    def gamma(self,element=None, edge=None, **kw):
        """Return an approximation of the core-hole lifetime for an element and edge.
        Data comes from K. Rahkonen and K. Krause, Atomic Data and Nuclear Data
        Tables, Vol 14, Number 2, 1974. 

        For O and P edges a value of 0.1 is returned

        Returns 0 for non-interpretable input
        """

        if edge is None: return None
        return self.server.gamma(self._prep(element=element, edge=edge, **kw))

    def fluor_yield(self,element=None, edge=None, **kw):
        """Return the fluorescence yield for an atomic symbol and edge
   
        fyield = VC.fluor_yield('Cu', edge='K')
        
        The value returned is the probability of an fluorescent x-ray being emitted
        for an absorption event.   
        Data comes from M. O. Krause, J. Phys. Chem. Ref. Data 8, 307 (1979)
        Returns -1 for non-interpretable input
        """

        if edge is None: return None
        return self.server.fluor_yield(self._prep(element=element, edge=edge, **kw))

    def edge_jump(self,element=None, edge=None, **kw):
        """Return edge jump ratio for an atomic symbol and edge

        jump = VC.edge_jump('Cu', edge='K')

        The value returned is the ratio of the above-edge absorption coefficient 
        to the below-edge coefficient
        """

        if edge is None: return None
        return self.server.edge_jump(self._prep(element=element, edge=edge, **kw))

    def intensity(self,element=None, line=None, **kw):
        """Return the intensity of the given emission line.  The
         intensities of all lines associated with a common core state sum to one.
         For example, the intensities of the 3 Kalpha and 5 Kbeta lines sum to one. 
        """

        if line is None:
            print 'need an line'
        return self.server.intensity(self._prep(element=element, line=line, **kw))

    
    def xsec(self,**kw):
        "total cross-section"
        return self.server.xsec(self._prep(**kw))

    def photo(self,**kw):
        "photo cross-section"
        return self.server.photo(self._prep(**kw))

    def coherent(self,**kw):
        "coherent cross-section"
        return self.server.coherent(self._prep(**kw))

    def incoherent(self,**kw):
        "incoherent cross-section"
        return self.server.incoherent(self._prep(**kw))

    def f1(self,resource='CL',**kw):
        "f' from CL "
        out =self.server.f1(self._prep(resource='CL',**kw))
        self.state_kws['resource'] = None
        return out

    def f2(self,resource='CL',**kw):
        "f'' from CL  "
        out =self.server.f2(self._prep(resource='CL',**kw))
        self.state_kws['resource'] = None
        return out

# $daemon -> dispatch_to(qw(has weight density conversion gamma
# 			  siegbahn iupac intensity
# 			  energy next_energy
# 			  xsec photo coherent incoherent f1 f2));

        

# print s.next_energy({'element': "Ti",'edge': "K",'atoms':['ba','ti','o']})        

if __name__ == '__main__':

    vc = VulcanClient(port=13702)
    print vc.available
    vc.has(element='Cu')
    print vc.weight()
    print vc.energy(edge='k')
    print vc.energy(line='kalpha1')
    print vc.energy(edge='k')
    print vc.gamma(edge='k')

    vc.has(element='Mo')
    print vc.weight()
    print vc.energy(edge='k')
    print vc.xsec(energies=[19994,20006])

#             

# 
# kws ={'element':"Cu", 'edge':'k'}
# 
# print s.has(element='Cu',edge='K')
# 
# print s.weight(kws), s.density(kws)
# 
# print s.energy({'element':'Cu', 'edge':'k'})
# print s.energy({'element':'Cu', 'line':'kalpha1'})
# 
# print "Core-hole lifetime of %(element)s K edge = " % kws,
# 
# print s.gamma(kws)
# 
# print s.next_energy({'element': "Ti",'edge': "K",'atoms':['ba','ti','o']})
# 

# print "elem Kalpha1 line = ", $rpcdata->result, $/;

# $rpcdata = $client -> iupac({line=>"lbeta3"});
# print "IUPAC notation for Lbeta3 = ", $rpcdata->result, $/;
# 
# 
# print $/;
# print "Elam total cross sections at 8700, 8800, 8900, 9000, 9100, 9200:\n";
# $rpcdata = $client -> xsec({element  => $elem,
# 			    energies => [8700, 8800, 8900, 9000, 9100, 9200]}
# 		      );
# print join(" ",@{ $rpcdata->result }), $/;
# 
# my $sum = 0;
# $rpcdata = $client -> intensity({element => $elem,
# 				 line    => "kalpha1"});
# print "$elem Kalpha1 intensity = ", $rpcdata->result, $/;
# $sum += $rpcdata->result;
# $rpcdata = $client -> intensity({element => $elem,
# 				 line    => "kalpha2"});
# print "$elem Kalpha2 intensity = ", $rpcdata->result, $/;
# $sum += $rpcdata->result;
# $rpcdata = $client -> intensity({element => $elem,
# 				 line    => "kalpha3"});
# print "$elem Kalpha3 intensity = ", $rpcdata->result, $/;
# $sum += $rpcdata->result;
# $rpcdata = $client -> intensity({element => $elem,
# 				 line    => "kb1"});
# print "$elem Kbeta1 intensity = ", $rpcdata->result, $/;
# $sum += $rpcdata->result;
# $rpcdata = $client -> intensity({element => $elem,
# 				 line    => "kb2"});
# print "$elem Kbeta2 intensity = ", $rpcdata->result, $/;
# $sum += $rpcdata->result;
# $rpcdata = $client -> intensity({element => $elem,
# 				 line    => "kb3"});
# print "$elem Kbeta3 intensity = ", $rpcdata->result, $/;
# $sum += $rpcdata->result;
# $rpcdata = $client -> intensity({element => $elem,
# 				 line    => "kb4"});
# print "$elem Kbeta4 intensity = ", $rpcdata->result, $/;
# $sum += $rpcdata->result;
# $rpcdata = $client -> intensity({element => $elem,
# 				 line    => "kb5"});
# print "$elem Kbeta5 intensity = ", $rpcdata->result, $/;
# $sum += $rpcdata->result;
# print $sum;
# 
# $rpcdata = $client -> incoherent({element  => $elem,
# 				  energies => [8700, 8800, 8900, 9000, 9100, 9200],
# 				  resource => "McMaster",
# 				 } );
# 
# print $/;
# print "McMaster incoherent cross sections at 8700, 8800, 8900, 9000, 9100, 9200:\n";
# print join(" ",@{ $rpcdata->result }), $/;
# 
# $rpcdata = $client -> intensity({element  => $elem,
# 				 line     => "kalpha1",
# 				 resource => "McMaster"});
# print "$elem Kalpha1 intensity = ", $rpcdata->result, $/;
# $rpcdata = $client -> intensity({element  => $elem,
# 				 line     => "kalpha2",
# 				 resource => "McMaster"});
# print "$elem Kalpha2 intensity = ", $rpcdata->result, $/;
# 
# 
# 
# $rpcdata = $client -> energy({element => "Ti",
# 			      edge    => "k"});
# print "\nTi K edge = ", $rpcdata->result, $/;
# print "next energy = ", join(" ", @{$rpcdata->result}), $/;



#!/usr/bin/env python
"""
Web Vulcan: a web-based interface to Vulcan.

This relies on mod_python and VulcanClient

"""

from mod_python import apache, Cookie
import time
import os

from VulcanClient import VulcanClient
FormAction = "http://cars9.uchicago.edu/~newville/py/webvulcan.py"

from HTMLData import CSSData, PTable,make_ptable

class WebVulcan:
    def __init__(self,port=None):
        self.buffer = []
        self.vc     = VulcanClient(port=port)
        self.port   = self.vc.port
        self.elem   = ''
        self.title  = 'X-ray Properties of the Elements'
        self.energy_in  = 30000.0
        self.energy_min =  1000.0

    def write(self,s):
        if s is None: s = '<None>'
        self.buffer.append(s)
        return
    
    def _get_attr(self,attr,**kw):
        elem = kw.get('elem',self.elem)
        return getattr(self.vc,attr)(element=elem, **kw)        
    
    def has(self,**kw):        return self._get_attr('has',**kw)
    def weight(self,**kw):     return self._get_attr('weight',**kw)
    def density(self,**kw):    return self._get_attr('density',**kw)
    def energy(self,**kw):     return self._get_attr('energy',**kw)
    def gamma(self,**kw):      return self._get_attr('gamma',**kw)
    def edge_jump(self,**kw):  return self._get_attr('edge_jump',**kw)
    def fluor_yield(self,**kw): return self._get_attr('fluor_yield',**kw)
    def intensity(self,**kw):  return self._get_attr('intensity',**kw)
    def siegbahn(self,**kw):   return self._get_attr('siegbahn',**kw)
    def iupac(self,**kw):      return self._get_attr('iupac',**kw)
    def xsec(self,**kw):       return self._get_attr('xsec',**kw)
    def photo(self,**kw):      return self._get_attr('photo',**kw)
    def coherent(self,**kw):   return self._get_attr('coherent',**kw)
    def incoherent(self,**kw): return self._get_attr('incoherent',**kw)    
    
    def at_name(self,sym=None):
        _dat = ['unknown','hydrogen','helium','lithium','beryllium','boron','carbon',
                'nitrogen','oxygen','fluorine','neon','sodium','magnesium',
                'aluminum','silicon','phosphorus','sulfur','chlorine','argon',
                'potassium','calcium','scandium','titanium','vanadium','chromium',
                'manganese','iron','cobalt','nickel','copper','zinc','gallium',
                'germanium','arsenic','selenium','bromine','krypton','rubidium',
                'strontium','yttrium','zirconium','niobium','molybdenum',
                'technetium','ruthenium','rhodium','palladium','silver','cadmium',
                'indium','tin','antimony','tellurium','iodine','xenon','cesium',
                'barium','lanthanum','cerium','praseodymium','neodymium','promethium',
                'samarium','europium','gadolinium','terbium','dysprosium',
                'holmium','erbium','thulium','ytterbium','lutetium','hafnium',
                'tantalum','tungsten','rhenium','osmium','iridium','platinum',
                'gold','mercury','thallium','lead','bismuth','polonium','astatine',
                'radon','francium','radium','actinium','thorium','protactinium',
                'uranium','neptunium','plutonium','americium','curium','berkelium',
                'californium','einsteinium','fermium','mendelevium','nobelium','lawerencium']

        if sym is None: sym = self.elem
        if isinstance(sym,str):
            iz = self.at_number(sym)
        elif isinstance(sym,int):
            iz = sym
        else:
            iz = 0
        if iz < 0 or iz >= len(_dat): iz = 0
        return _dat[iz].title()


    def at_number(self,sym=None):
        _dat = [None,'H','He','Li','Be','B','C','N','O','F','Ne',
               'Na','Mg','Al','Si','P','S','Cl','Ar','K','Ca','Sc',
               'Ti','V','Cr','Mn','Fe','Co','Ni','Cu','Zn','Ga','Ge',
               'As','Se','Br','Kr','Rb','Sr','Y','Zr','Nb','Mo','Tc',
               'Ru','Rh','Pd','Ag','Cd','In','Sn','Sb','Te','I','Xe',
               'Cs','Ba','La','Ce','Pr','Nd','Pm','Sm','Eu','Gd','Tb',
               'Dy','Ho','Er','Tm','Yb','Lu','Hf','Ta','W','Re','Os',
               'Ir','Pt','Au','Hg','Tl','Pb','Bi','Po','At','Rn','Fr',
               'Ra','Ac','Th','Pa','U','Np','Pu','Am','Cm','Bk','Cf',
               'Bk','Cf','Es','Fm','Md','No','Lr']
        
        if sym is None: sym = self.elem
        try:
            return _dat.index(sym.strip().title())
        except:
            return 0
        
    def start_html(self):
        self.write('<head><title>%s:</title></head><body>' % self.title)
        self.write(CSSData)
        
    def get_html(self):
        out = '\n'.join(self.buffer)
        self.buffer = []
        return out

    def show_emissiontable(self,edge,lineslist):

        # don't show edges above excitation energy
        edge_energy = self.energy(edge=edge)
        if (self.energy_in < edge_energy) or (self.energy_min > edge_energy):
            return
        
        self.write("<table cellpadding=3><tr><td colspan=5></td></tr>")

        self.write("<tr><td colspan=5>%s-Edge (E = %.1f) Emission Lines</td></tr>" % (edge.title(),edge_energy))

        self.write("""<tr><td></td><td>Line</td><td>Transition</td><td>Energy&nbsp;</td><td>Strength</td>
        </tr><tr><td>&nbsp;&nbsp;</td><td colspan=4><hr></td></tr>""")#  % edge.title())

        eupper = edge.upper()
        for lines in lineslist:
            needspace = False
            for key,en,iupac,int in lines:
                if en > 1 and int > 0.0005 and iupac.startswith(eupper):
                    sty = ''
                    if en < self.energy_min:  sty = 'id=grayentry'
                    self.write("""<tr %s><td></td><td>%s</td><td>%s</td><td align=right>%.1f</td>
                    <td align=right>%.3f</td></tr>""" % (sty,key,iupac,en,int))
                    # needspace = True
            #if needspace and lines != lineslist[-1]:
            #    self.write("<tr><td colspan=2>&nbsp;&nbsp;</td><td colspan=2></td></tr>")

        self.write("<tr><td>&nbsp;&nbsp;</td><td colspan=4><hr></td></tr></table>")

    def show_energy_data(self):
        def line(s):  return (self.siegbahn(line=s),self.energy(line=s),self.iupac(line=s),self.intensity(line=s))
        def edge(s):  return {'title':s.title(),'energy':self.energy(edge=s),
                              'gamma':self.gamma(edge=s), 'jump': self.edge_jump(edge=s),
                              'fyield': self.fluor_yield(edge=s)}


        kalphas = (line('ka1'),line('ka2'),line('ka3'))
        kbetas  = (line('kb1'),line('kb2'),line('kb3'),line('kb4'),line('kb5'))


        lalphas = (line('la1'),line('la2'))
        lbetas  = (line('lb1'),line('lb2'),line('lb3'),line('lb4'),line('lb5'),line('lb6'))
        lgammas = (line('lg1'),line('lg2'),line('lg3'),line('lg4'),line('lg5'),line('lg6'))
        lothers = (line('ll'),line('lnu'))
        mlines  = (line('ma'),line('mb'),line('mg'),line('mz'))

        sym = self.elem
        ename = self.at_name()
        iz = self.at_number()
        atwt  = self.weight()
        self.write("""<h4>%s: %s &nbsp;&nbsp;
        (Z=%i) &nbsp;&nbsp;  Atomic Weight=%.1f</h4>\n""" % (ename,sym,iz,atwt))
        
        self.write("""<table cellpadding=2><tr valign=top><td width=35%%>
        <h4>Excitation Energies (eV):</h4>
        <table><tr><td>Edge&nbsp;</td><td>Energy&nbsp;</td><td>Width&nbsp;</td>
        <td>Edge Jump&nbsp;</td><td>Fluor Yield&nbsp;</td></tr>
        <tr><td colspan=5><hr></td></tr>
        """)
        format = """<tr><td>%(title)s</td><td align=right>%(energy).1f</td>
        <td align=right>%(gamma).2f</td><td align=center>%(jump).2f</td><td align=center>%(fyield).3f</td>"""

        for e in ('k','l3','l2','l1'):  self.write(format % edge(e))        
                   

        self.write("""<tr><td colspan=5><hr></td></tr></table>
        </td><td width=60%%>
        <h4>Emission Energies (eV):</h4>""")
        
        self.show_emissiontable('k',  [kalphas,kbetas])
        self.show_emissiontable('l3', [lalphas,lbetas,lgammas,lothers])
        self.show_emissiontable('l2', [lalphas,lbetas,lgammas,lothers])
        self.show_emissiontable('l1', [lalphas,lbetas,lgammas,lothers])        
        self.write("</td></tr></table>")
        

    def draw_ptable(self):
        wr = self.write
        wr("WebVulcan 0.1:  &nbsp; X-ray Excitation and Emission Energies for Elements<p><hr>")
        # wr(PTable % (self.energy_max,self.energy_min,self.elem))
        linkargs = '&energy_in=%.1f&energy_min=%1.f' % (self.energy_in, self.energy_min)
        make_ptable(writer=wr, form_action=FormAction,linkargs=linkargs,
                    form_elem=self.elem, energy_max = self.energy_in,
                    energy_min = self.energy_min)
   

def connect(req):
    """connect to a WebVulcan instance, including a port to a VulcanServer
    uses a web cookie to hold the Port to the VulcanServer in state
    
    """

    # look for cookie, use held port if available
    has_cookie = False
    port = None
    try:
        cookies= Cookie.get_cookies(req, Cookie.MarshalCookie, secret='foo')
        if cookies.has_key('VulcanPort'):
            cookie = cookies['VulcanPort']
            port = int(cookie.value)
            if port < 12000: port = None
            has_cookie = True
    except:
        pass
        
    # create a WebVulcan instance with this port
    w    = WebVulcan(port=port)

    # if needed, save this port in a cookie to be read next time
    if not has_cookie:
        cookie = Cookie.MarshalCookie('VulcanPort',w.port,'foo')
        cookie.expires = time.time() + 1800
        Cookie.add_cookie(req,cookie)

    return w
    
def isnull(s):
    if isinstance(s,str): s = s.strip()
    return s in ('','None',None,False,'False')
    
def energies(req, elem='', submit='',debug=False,
          energy_in=None,energy_min=None, form_elem='',   **kw):
    """main interaction for WebVulcan
    """

    # connect to WebVulcan
    c = connect(req)

    atname = c.at_name()
    if atname != 'unknown':
        c.title = '%s - %s' % (c.title,atname)

    c.start_html()

    if not isnull(debug):
        c.write("Args<br>")
        args = locals()
        for k in ('elem','form_elem','submit','energy_in','energy_min'):
            v = args.get(k,'')
            c.write("%s = '%s'<br>" % (k,v))

        c.write("Extr Args<br>")
        for k,v in kw.items():
            c.write("%s = '%s'<br>" % (k,v))


    if isnull(elem) and not isnull(form_elem):       elem = form_elem

    if not isnull(elem) and isinstance(elem,str):    c.elem = elem.strip()

    if isnull(c.elem) and not isnull(form_elem):
        c.elem = form_elem.strip()

    try:
        c.energy_in = float(energy_in)
    except:
        pass
    try:
        c.energy_min = float(energy_min)
    except:
        pass


    c.draw_ptable()
    c.write("""<table><tr><td>Excitation Energy (eV):</td>
    <td><input name='energy_in' type=text value='%.1f' size=13></td><td></td></tr>
    <tr><td>Minimum Energy(eV):</td>
    <td><input name='energy_min' type=text value='%.1f' size=13></td>
    <td> (lowest energy reported)</td></tr>
    <tr><td><input type=submit  name='Submit'      value='Submit'> </td><td></td></tr>
    </table>
    <input type=hidden name='form_elem'    value='%s'>
    </form>
    """ % (c.energy_in,c.energy_min,c.elem))

    if c.elem != '':
        c.show_energy_data()
    c.write('<hr></body></html>')
    return c.get_html()



def index(req,**kw):
    return energies(req,**kw)
